#include "RSA-core.hpp"

RSA::RSA ()
{
  mpz_init (p);
  mpz_init (q);
  mpz_init (n);
  mpz_init (phin);
  mpz_init (s);
  mpz_init (e);

  mpz_init (x);
  mpz_init (y);
};

RSA::~RSA ()
{
  mpz_clear (y);
  mpz_clear (x);
  
  mpz_clear (e);
  mpz_clear (s);
  mpz_clear (phin);
  mpz_clear (n);
  mpz_clear (q);
  mpz_clear (p);
};

void RSA::calc_n_from_p_q ()
{
  mpz_mul (n, p, q);
};

void RSA::calc_phin_from_p_q ()
{
  mpz_sub_ui (x, p, 1);
  mpz_sub_ui (y, q, 1);
  mpz_mul (phin, x, y);
};

void RSA::calc_e_from_s_phin ()
{
  /*  s*e + phin*d = g = ggT(s,phin)  all integer  */
  mpz_gcdext (x, e, 0, s, phin); // calculate e
  mpz_tdiv_r (e, e, phin); // adjust e modulo phin
  if (mpz_sgn (e) < 0) mpz_add (e, e, phin); // if e negative, add phin
};

void RSA::calc_b_from_a_s_n (mpz_t b, mpz_t a)
{
  mpz_powm (b, a, s, n);
};

void RSA::calc_a_from_b_e_n (mpz_t a, mpz_t b)
{
  mpz_powm (a, b, e, n);
};
