#ifndef __RSA_CORE_HPP
#define __RSA_CORE_HPP

#include <gmp.h>

/*

  definition of the used RSA Algorithm:

  n, s  are the public key
  n, e  are the secret key
  a is the plain message
  b is the encrypted message

  if {

    p, q are prime numbers. ( >1 )
    p != q
    n = p * q
    phi(n) = (p-1) * (q-1)

    0 < s < phi(n)
    0 < e < phi(n)
    gcd (s, phi(n)) = 1
    gcd (e, phi(n)) = 1
    (s * e) mod phi(n) = 1

    0 <= a < n
    0 <= b < n

  } then {

    b = a^s mod n
    a = b^e mod n

  }

  You may only use values that are accepted by the if block above !
  Otherwise the behaviour is undefined. It may even crash !

  The class RSA implements correct RSA.
  it is not said, that this is "secure" !
  it is just correct RSA.
  it is up to you to feed the routines with "secure" values!

*/

class RSA {
public:
  RSA ();
  RSA (const RSA &);             // mpz_t mustn't be just copied
  RSA & operator= (const RSA &); // mpz_t mustn't be just copied
  ~RSA ();

  void calc_n_from_p_q ();
  void calc_phin_from_p_q ();
  void calc_e_from_s_phin ();
  void calc_b_from_a_s_n (mpz_t b, mpz_t a);
  void calc_a_from_b_e_n (mpz_t a, mpz_t b);

  mpz_t p, q, n, phin, s, e;

private:
  mpz_t x, y;
};

#endif
