#include <stdio.h>

#include "decipher.hpp"
#include "common.hpp"
#include "countbits.hpp"
#include "RSA-core.hpp"
#include "numberio.hpp"

#include <stdlib.h>

int decipher (int argc, char *argv[])
{
  if (argc!=5) usage ();

  RSA rsa;

  mpz_t remainder, a, b;
  mpz_init (remainder);
  mpz_init (a);
  mpz_init (b);

  FILE *inputfile, *outputfile, *secretfile;

  assert ((secretfile = fopen (argv[4], "r"))!=0);
  const char header[] = "yyyRSA SECRET KEY V2";
  char header_checker [sizeof(header)-1];
  assert2 (fread(header_checker, sizeof(header)-1, 1, secretfile)==1,
	   "the secret file is no yyyRSA SECRET KEY V2 file.");
  assert2 (memcmp(header_checker, header, sizeof(header)-1)==0,
	   "the secret file is no yyyRSA SECRET KEY V2 file.");
  number_read (secretfile, rsa.n);
  number_read (secretfile, rsa.e);
  assert (fclose(secretfile)==0);

  assert ((inputfile = fopen (argv[2], "r"))!=0);
  long inputlength;
  fscanf (inputfile, "%ld", &inputlength);
  //  printf ("length of input file %ld bytes\n", inputlength);

  assert ((outputfile = fopen (argv[3], "w"))!=0);

  int totalbits = countbits(rsa.n) - 1; // real usable bits in keyspace
  int databits = totalbits >> 1;
  int bytes = databits >> 3;
  assert2 (bytes>0, "key is too small to encipher a byte, this is probably a corrupt secret key.");
  unsigned char *buffer = (unsigned char *) malloc(bytes);
  assert2 (buffer, "memory full");
  printf ("now deciphering. Each dot is %d bytes ", bytes);
  fflush (stdout);

  while (inputlength) {
    printf("."); fflush (stdout);
    long todo = inputlength > bytes ? bytes : inputlength;
    
    number_read (inputfile, b);
    
    rsa.calc_a_from_b_e_n (a, b);

    for (long i=0;i<todo;i++) {
      mpz_tdiv_qr_ui (a, remainder, a, 256);
      buffer[i]=mpz_get_ui (remainder);
    };

    assert (fwrite (buffer, 1, todo, outputfile)==(unsigned long)todo);

    inputlength -= todo;
  };

  printf(" done\n");

  free (buffer);

  assert (ferror(outputfile)==0);
  assert (ferror(inputfile)==0);
  assert (fclose(outputfile)==0);
  assert (fclose(inputfile)==0);

  mpz_clear (b);
  mpz_clear (a);
  mpz_clear (remainder);

  return 0;
};
