#include <stdio.h>

#include "encrypt.hpp"
#include "common.hpp"
#include "countbits.hpp"
#include "RSA-core.hpp"
#include "gen_random.hpp"
#include "numberio.hpp"

#include <stdlib.h>

int encrypt (int argc, char *argv[])
{
  if (argc!=5) usage ();

  RSA rsa;
  mpz_t a, b;
  mpz_init (a);
  mpz_init (b);

  FILE *inputfile, *outputfile, *publicfile;

  assert ((publicfile = fopen (argv[4], "r"))!=0);
  const char header[] = "yyyRSA PUBLIC KEY V2";
  char header_checker [sizeof(header)-1];
  assert2 (fread(header_checker, sizeof(header)-1, 1, publicfile)==1,
	   "the public file is no yyyRSA PUBLIC KEY V2 file.");
  assert2 (memcmp(header_checker, header, sizeof(header)-1)==0,
	   "the public file is no yyyRSA PUBLIC KEY V2 file.");
  number_read (publicfile, rsa.n);
  number_read (publicfile, rsa.s);
  assert (fclose(publicfile)==0);
  
  assert ((inputfile = fopen (argv[2], "r"))!=0);
  fseek (inputfile, 0, SEEK_END);
  long inputlength = ftell (inputfile);
  rewind (inputfile);
  //  printf ("length of input file %ld bytes\n", inputlength);

  assert ((outputfile = fopen (argv[3], "w"))!=0);
  fprintf (outputfile, "%ld\n", inputlength);

  int totalbits = countbits(rsa.n) - 1; // real usable bits in keyspace
  int databits = totalbits >> 1;
  int randombits = totalbits - databits;
  int bytes = databits >> 3;
  assert2 (bytes>0, "key is too small to encipher a byte :)");
  unsigned char *buffer = (unsigned char *) malloc(bytes);
  assert2 (buffer, "memory full");
  printf ("now encrypting. Each dot is %d bytes ", bytes);
  fflush (stdout);

  while (inputlength) {
    printf("."); fflush (stdout);
    long todo = inputlength > bytes ? bytes : inputlength;
    assert (fread (buffer, 1, todo, inputfile)==(unsigned long)todo);

    if (todo < bytes) {
      // we are at the last block.
      // better pad it with entropy to make it a full block!
      // long live paranoia !
      gen_deterministic_random(randombits + databits - (todo<<3), a);
    }else{
      // a normal, full block.
      gen_deterministic_random(randombits, a); // fill with entropy !
    };

    for (long i=todo-1;i>=0;i--) {
      mpz_mul_2exp (a, a, 8);
      mpz_add_ui (a, a, buffer[i]);
    };

    assert2 (mpz_cmp (a, rsa.n) < 0, "a >= n  WILL NOT WORK. ALGORITHM ERROR\n");
    rsa.calc_b_from_a_s_n (b, a);

    number_write (outputfile, b);

    inputlength -= todo;
  };

  printf(" done\n");

  free (buffer);

  assert (ferror(outputfile)==0);
  assert (ferror(inputfile)==0);
  assert (fclose(outputfile)==0);
  assert (fclose(inputfile)==0);

  mpz_clear (b);
  mpz_clear (a);

  return 0;
};
