#include "numberio.hpp"

#include "common.hpp"

static char codec[] = { // codec has 64 elements.
  'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 
  'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 
  'y', 'z',
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 
  'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 
  'Y', 'Z',
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  '+', '/'
};

static inline const char encode (const unsigned char value)
{
  assert (value<64);
  return codec[value];
};

static inline const unsigned char decode (const char value)
{
  for (int i=0; i<64; ++i)
    if (codec[i]==value) return i;
  assert2(false, "illegal characters to be decoded.");
};

void number_write (FILE *file, mpz_t value)
{
  assert (mpz_cmp_si (value, 0) >= 0);

  mpz_t copy, res;
  mpz_init_set (copy, value);
  mpz_init (res);
  int line=-1;
  
  do {
    mpz_tdiv_r_2exp (res, copy, 6);
    if (++line==70) {
      line=0;
      fprintf(file,"=\n");
    };
    fprintf(file, "%c", encode (mpz_get_ui (res)));
    mpz_tdiv_q_2exp (copy, copy, 6);
  } while (mpz_cmp_ui(copy,0) != 0);

  fprintf(file, "\n");
  
  mpz_clear (res);
  mpz_clear (copy);
};

void number_read (FILE *file, mpz_t value)
{
  mpz_set_ui (value, 0);
  char c;
  mpz_t res;
  mpz_init (res);
  int i=0;
  
  for (;;) {
    assert (fread (&c, 1, 1, file)==1);
    if (c!=10) break;
  };
    
  for (;;) {
    if (c==10) break;
    if (c=='=') {
      assert (fread (&c, 1, 1, file)==1);
      assert (fread (&c, 1, 1, file)==1);
      continue;
    };
    mpz_set_ui (res, decode (c));
    mpz_mul_2exp (res, res, 6*i);
    mpz_add (value, value, res);
    assert (fread (&c, 1, 1, file)==1);
    ++i;
  };

  mpz_clear (res);
};
